==> wm-react-native-components-0.0.63.tgz: package/recyclerview/RecyclerView.android.js <==
/**
 * Created with IntelliJ IDEA.
 * User: aaronlin
 * Date: 2018/5/14
 * Time: 17:46
 */

import React, { PureComponent } from 'react';
import ReactNative, {
    View,
    Text,
    Platform,
    requireNativeComponent,
    UIManager,
} from 'react-native';
import PropTypes from 'prop-types';

import RecyclerItemView from './RecyclerItemView';

/**
 *
 */
const NativeRecyclerView = requireNativeComponent('AndroidRecyclerView', RecyclerView);

/**
 *
 */
const NativeAndroidRecyclerFooterRefreshContainerView = requireNativeComponent('AndroidRecyclerFooterRefreshContainerView');

/**
 *
 */
const NativeAndroidRecyclerFooterEndRefreshContainerView = requireNativeComponent('AndroidRecyclerFooterEndRefreshContainerView');

/**
 *
 */
const NativeAndroidRecyclerHeaderContainerView = requireNativeComponent('AndroidRecyclerHeaderContainerView');

/**
 *
 */
const CONSTANTS_SCROLL_STATE = UIManager.AndroidRecyclerView.Constants.SCROLL_STATE;

/**
 *
 */
export default class RecyclerView extends PureComponent{

    /**
     *
     * @type {{dataList: *, requestLayoutIfScroll: React$PropType$Primitive.<boolean>}}
     */
    static propTypes = {
        dataList:PropTypes.array.isRequired,    //数据列表
        requestLayoutIfScroll:PropTypes.bool,   //在 scroll 时是否重新requestLayout
        itemLayoutAnimationEnabled:PropTypes.bool,   //itemview 是否启用 LayoutAnimation
    }

    /**
     *
     * @type {{dataList: Array, requestLayoutIfScroll: boolean}}
     */
    static defaultProps = {
        dataList:[],
        requestLayoutIfScroll:false,
        itemLayoutAnimationEnabled:false,
    }

    /**
     *
     * @type {null}
     * @private
     */
    _scrollEventObj = null;

    /**
     *
     * @type {Array}
     * @private
     */
    _dataList = [];

    constructor(props){
        super(props);

        this._dataList.addAll(this.props.dataList);

        this._initScrollEventIfEmpty();
    }

    /**
     *
     * @private
     */
    _initScrollEventIfEmpty = ()=>{
        if(this._scrollEventObj == null){
            this._scrollEventObj = {
                nativeEvent: {
                    contentInset: {
                        bottom: 0,
                        top: 0,
                        left: 0,
                        right: 0
                    },
                    contentOffset:{
                        x:0,
                        y:0,
                        dx:0,
                        dy:0,
                    },
                    posState:{
                        firstPosition:0,
                        isTop:true,
                        isBottom:false,
                    }
                }
            };
        }
    }

    /**
     * newState:
     * SCROLL_STATE_IDLE = 0
     * SCROLL_STATE_DRAGGING = 1
     * SCROLL_STATE_SETTLING = 2
     * @param e
     * @private
     */
    _onScrollStateChanged = (e)=>{
        let { newState, posState } = e.nativeEvent;

        const { onScrollBeginDrag, onScrollEndDrag } = this.props;

        if(newState === CONSTANTS_SCROLL_STATE.SCROLL_STATE_DRAGGING){    //开始
            this._initScrollEventIfEmpty();
            Object.assign(this._scrollEventObj.nativeEvent.posState, posState);
            onScrollBeginDrag && onScrollBeginDrag(this._scrollEventObj);
        }else if(newState === CONSTANTS_SCROLL_STATE.SCROLL_STATE_IDLE){  //停止
            onScrollEndDrag && onScrollEndDrag(this._scrollEventObj);
            this._initScrollEventIfEmpty();
        }
    }

    /**
     *
     * @param e
     * @private
     */
    _onScrolled = (e)=>{
        const { nativeEvent } = e;

        const { nativeEvent:scrollNativeEvent } = this._scrollEventObj;
        Object.assign(scrollNativeEvent, nativeEvent);

        const { onScroll } = this.props;

        onScroll && onScroll(this._scrollEventObj);
    }

    /**
     * ListView.smoothScrollToPositionFromTop 暂不支持水平滚动
     * @param position
     * @param offset
     * @param animated
     */
    smoothScrollToPositionFromTop = (position = 0, offset = 0, animated = true)=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.smoothScrollToPositionFromTop, [position, offset, animated]);
        }
    }

    /**
     * scrollBy
     * @param x
     * @param y
     */
    scrollBy = (x = 0, y = 0)=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.scrollBy, [x, y]);
        }
    }

    /**
     * notifyDataSetChanged
     */
    notifyDataSetChanged = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.notifyDataSetChanged, null);
        }
    }

    /**
     * stopScroll
     */
    stopScroll = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.stopScroll, null);
        }
    }

    /**
     * onLoadMoreComplete
     */
    onLoadMoreComplete = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.onLoadMoreComplete, null);
        }
    }

    /**
     * onLoadMoreCompleteAndNoData
     */
    onLoadMoreCompleteAndNoData = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.onLoadMoreCompleteAndNoData, null);
        }
    }

    /**
     * setItemCount
     */
    setItemCount = (itemCount : number)=>{
        if(itemCount >= 0){
            this.setNativeProps({
                itemCount:itemCount
            });
        }
    }

    /**
     * 刷新状态
     * @param e
     * @private
     */
    _onRefreshState = (e)=>{
        const{ onRefreshState } = this.props;
        onRefreshState && onRefreshState(e);
    }

    /**
     * 加载更多
     * @param e
     * @private
     */
    _onLoadMore = (e)=>{
        const{ onLoadMore } = this.props;
        onLoadMore && onLoadMore(e);
    }

    /**
     *
     * @private
     */
    _getListViewHandle = ()=>{
        if(this.refs.nativeRecyclerView){
            return ReactNative.findNodeHandle(this.refs.nativeRecyclerView);
        }
    }

    /**
     *
     * @param props
     */
    setNativeProps = (props)=>{
        this.refs.nativeRecyclerView && this.refs.nativeRecyclerView.setNativeProps(props);
    }

    render(){
        const { dataList, renderHeader, renderRow, renderSeparator, initialListSize = 15, footerRefreshView, footerEndView, ...otherProps } = this.props;

        let itemViews = [];

        if(footerRefreshView){
            let refreshFooterEndContainerView = null;
            if(footerEndView){
                refreshFooterEndContainerView = (
                    <NativeAndroidRecyclerFooterEndRefreshContainerView>
                        {footerEndView}
                    </NativeAndroidRecyclerFooterEndRefreshContainerView>
                );
            }

            let nativeLoadMoreView = (
                <NativeAndroidRecyclerFooterRefreshContainerView
                    key={'nativeAndroidRecyclerFooterRefreshContainerView'}
                >
                    {footerRefreshView}
                    {refreshFooterEndContainerView}
                </NativeAndroidRecyclerFooterRefreshContainerView>
            );
            itemViews.add(nativeLoadMoreView);
        }

        //头部
        if(renderHeader){
            let _renderHeaderView = renderHeader();
            if(_renderHeaderView){
                const { children, ...otherProps } = _renderHeaderView.props;
                if(Object.isEmpty(otherProps)){
                    _renderHeaderView = React.cloneElement(_renderHeaderView, {
                        testID:'renderHeaderViewTestID',
                    });
                }
            }
            let headerView = (
                <NativeAndroidRecyclerHeaderContainerView
                    key={'nativeAndroidRecyclerHeaderContainerView'}
                >
                    {_renderHeaderView}
                </NativeAndroidRecyclerHeaderContainerView>
            );
            itemViews.add(headerView);
        }

        let itemCount = dataList.length;
        if(dataList && itemCount > 0 && renderRow){
            for(let index = 0; index < Math.min(itemCount, initialListSize); index++){
                // for(let index = 0; index < initialListSize; index++){
                let recyclerItemView = (
                    <RecyclerItemView
                        key={'nativeRecyclerItemView-'+index}
                        position={index}
                        dataList={dataList}
                        renderRow={renderRow}
                        renderSeparator={renderSeparator}
                    />
                );
                itemViews.add(recyclerItemView);
            }
        }

        let key;
        if(itemCount >= initialListSize){
            key = 'nativeRecyclerView-'+initialListSize;
        }else{
            key = 'nativeRecyclerView-'+itemCount;
        }

        return (
            <NativeRecyclerView
                ref="nativeRecyclerView"
                key={key}
                {...otherProps}
                itemCount={dataList.length}
                itemViewCacheSize={2}

                onRefreshState={this._onRefreshState}
                onLoadMore={this._onLoadMore}
                onScrolled={this._onScrolled}
                onScrollStateChanged={this._onScrollStateChanged}
            >

                { itemViews }

            </NativeRecyclerView>
        );
    }

}

==> wm-react-native-components-0.0.81.tgz: package/recyclerview/RecyclerView.android.js <==
/**
 * Created with IntelliJ IDEA.
 * User: aaronlin
 * Date: 2018/5/14
 * Time: 17:46
 */

import React, { PureComponent } from 'react';
import ReactNative, {
    View,
    Text,
    Platform,
    requireNativeComponent,
    UIManager,
} from 'react-native';
import PropTypes from 'prop-types';

import RecyclerItemView from './RecyclerItemView';

/**
 *
 */
const NativeRecyclerView = requireNativeComponent('AndroidRecyclerView', RecyclerView);

/**
 *
 */
const NativeAndroidRecyclerFooterRefreshContainerView = requireNativeComponent('AndroidRecyclerFooterRefreshContainerView');

/**
 *
 */
const NativeAndroidRecyclerFooterEndRefreshContainerView = requireNativeComponent('AndroidRecyclerFooterEndRefreshContainerView');

/**
 *
 */
const NativeAndroidRecyclerHeaderContainerView = requireNativeComponent('AndroidRecyclerHeaderContainerView');

/**
 *
 */
const CONSTANTS_SCROLL_STATE = UIManager.AndroidRecyclerView.Constants.SCROLL_STATE;

/**
 *
 */
export default class RecyclerView extends PureComponent{

    /**
     *
     * @type {{dataList: *, requestLayoutIfScroll: React$PropType$Primitive.<boolean>}}
     */
    static propTypes = {
        dataList:PropTypes.array.isRequired,    //数据列表
        requestLayoutIfScroll:PropTypes.bool,   //在 scroll 时是否重新requestLayout
        itemLayoutAnimationEnabled:PropTypes.bool,   //itemview 是否启用 LayoutAnimation
    }

    /**
     *
     * @type {{dataList: Array, requestLayoutIfScroll: boolean}}
     */
    static defaultProps = {
        dataList:[],
        requestLayoutIfScroll:false,
        itemLayoutAnimationEnabled:false,
    }

    /**
     *
     * @type {null}
     * @private
     */
    _scrollEventObj = null;

    /**
     *
     * @type {Array}
     * @private
     */
    _dataList = [];

    /**
     *
     */
    _lastKeyInt;

    constructor(props){
        super(props);

        this._dataList.addAll(this.props.dataList);

        this._initScrollEventIfEmpty();
    }

    /**
     *
     * @private
     */
    _initScrollEventIfEmpty = ()=>{
        if(this._scrollEventObj == null){
            this._scrollEventObj = {
                nativeEvent: {
                    contentInset: {
                        bottom: 0,
                        top: 0,
                        left: 0,
                        right: 0
                    },
                    contentOffset:{
                        x:0,
                        y:0,
                        dx:0,
                        dy:0,
                    },
                    posState:{
                        firstPosition:0,
                        isTop:true,
                        isBottom:false,
                    }
                }
            };
        }
    }

    /**
     * newState:
     * SCROLL_STATE_IDLE = 0
     * SCROLL_STATE_DRAGGING = 1
     * SCROLL_STATE_SETTLING = 2
     * @param e
     * @private
     */
    _onScrollStateChanged = (e)=>{
        let { newState, posState } = e.nativeEvent;

        const { onScrollBeginDrag, onScrollEndDrag } = this.props;

        if(newState === CONSTANTS_SCROLL_STATE.SCROLL_STATE_DRAGGING){    //开始
            this._initScrollEventIfEmpty();
            Object.assign(this._scrollEventObj.nativeEvent.posState, posState);
            onScrollBeginDrag && onScrollBeginDrag(this._scrollEventObj);
        }else if(newState === CONSTANTS_SCROLL_STATE.SCROLL_STATE_IDLE){  //停止
            onScrollEndDrag && onScrollEndDrag(this._scrollEventObj);
            this._initScrollEventIfEmpty();
        }
    }

    /**
     *
     * @param e
     * @private
     */
    _onScrolled = (e)=>{
        const { nativeEvent } = e;

        const { nativeEvent:scrollNativeEvent } = this._scrollEventObj;
        Object.assign(scrollNativeEvent, nativeEvent);

        const { onScroll } = this.props;

        onScroll && onScroll(this._scrollEventObj);
    }

    /**
     * ListView.smoothScrollToPositionFromTop 暂不支持水平滚动
     * @param position
     * @param offset
     * @param animated
     */
    smoothScrollToPositionFromTop = (position = 0, offset = 0, animated = true)=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.smoothScrollToPositionFromTop, [position, offset, animated]);
        }
    }

    /**
     * scrollBy
     * @param x
     * @param y
     */
    scrollBy = (x = 0, y = 0)=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.scrollBy, [x, y]);
        }
    }

    /**
     * notifyDataSetChanged
     */
    notifyDataSetChanged = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.notifyDataSetChanged, null);
        }
    }

    /**
     * stopScroll
     */
    stopScroll = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.stopScroll, null);
        }
    }

    /**
     * onLoadMoreComplete
     */
    onLoadMoreComplete = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.onLoadMoreComplete, null);
        }
    }

    /**
     * onLoadMoreCompleteAndNoData
     */
    onLoadMoreCompleteAndNoData = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidRecyclerView.Commands.onLoadMoreCompleteAndNoData, null);
        }
    }

    /**
     * setItemCount
     */
    setItemCount = (itemCount : number)=>{
        if(itemCount >= 0){
            this.setNativeProps({
                itemCount:itemCount
            });
        }
    }

    /**
     * 刷新状态
     * @param e
     * @private
     */
    _onRefreshState = (e)=>{
        const{ onRefreshState } = this.props;
        onRefreshState && onRefreshState(e);
    }

    /**
     * 加载更多
     * @param e
     * @private
     */
    _onLoadMore = (e)=>{
        const{ onLoadMore } = this.props;
        onLoadMore && onLoadMore(e);
    }

    /**
     *
     * @private
     */
    _getListViewHandle = ()=>{
        if(this.refs.nativeRecyclerView){
            return ReactNative.findNodeHandle(this.refs.nativeRecyclerView);
        }
    }

    /**
     *
     * @param props
     */
    setNativeProps = (props)=>{
        this.refs.nativeRecyclerView && this.refs.nativeRecyclerView.setNativeProps(props);
    }

    render(){
        const { dataList, renderHeader, renderRow, renderSeparator, initialListSize = 15, footerRefreshView, footerEndView, ...otherProps } = this.props;

        let itemViews = [];

        if(footerRefreshView){
            let refreshFooterEndContainerView = null;
            if(footerEndView){
                refreshFooterEndContainerView = (
                    <NativeAndroidRecyclerFooterEndRefreshContainerView>
                        {footerEndView}
                    </NativeAndroidRecyclerFooterEndRefreshContainerView>
                );
            }

            let nativeLoadMoreView = (
                <NativeAndroidRecyclerFooterRefreshContainerView
                    key={'nativeAndroidRecyclerFooterRefreshContainerView'}
                >
                    {footerRefreshView}
                    {refreshFooterEndContainerView}
                </NativeAndroidRecyclerFooterRefreshContainerView>
            );
            itemViews.add(nativeLoadMoreView);
        }

        //头部
        if(renderHeader){
            let _renderHeaderView = renderHeader();
            if(_renderHeaderView){
                const { children, ...otherProps } = _renderHeaderView.props;
                if(Object.isEmpty(otherProps)){
                    _renderHeaderView = React.cloneElement(_renderHeaderView, {
                        testID:'renderHeaderViewTestID',
                    });
                }
            }
            let headerView = (
                <NativeAndroidRecyclerHeaderContainerView
                    key={'nativeAndroidRecyclerHeaderContainerView'}
                >
                    {_renderHeaderView}
                </NativeAndroidRecyclerHeaderContainerView>
            );
            itemViews.add(headerView);
        }

        let itemCount = dataList.length;
        if(dataList && itemCount > 0 && renderRow){
            for(let index = 0; index < Math.min(itemCount, initialListSize); index++){
                // for(let index = 0; index < initialListSize; index++){
                let recyclerItemView = (
                    <RecyclerItemView
                        key={'nativeRecyclerItemView-'+index}
                        position={index}
                        dataList={dataList}
                        renderRow={renderRow}
                        renderSeparator={renderSeparator}
                    />
                );
                itemViews.add(recyclerItemView);
            }
        }

        let _currKeyInt;
        if(itemCount >= initialListSize){
            _currKeyInt = initialListSize;

            if(_currKeyInt === this._lastKeyInt && itemCount === initialListSize){   //下拉刷新
                _currKeyInt = new Date().getTime();
            }else if(itemCount > initialListSize){  //加载更多
                _currKeyInt = this._lastKeyInt;
            }
            this._lastKeyInt = _currKeyInt;

        } else {
            _currKeyInt = itemCount;
        }

        let key = 'nativeRecyclerView-'+_currKeyInt;

        return (
            <NativeRecyclerView
                ref="nativeRecyclerView"
                key={key}
                {...otherProps}
                itemCount={dataList.length}
                itemViewCacheSize={2}

                onRefreshState={this._onRefreshState}
                onLoadMore={this._onLoadMore}
                onScrolled={this._onScrolled}
                onScrollStateChanged={this._onScrollStateChanged}
            >

                { itemViews }

            </NativeRecyclerView>
        );
    }

}
